/*****************************************************
This program was produced by the
CodeWizardAVR V1.25.9 Professional
Automatic Program Generator
 Copyright 1998-2008 Pavel Haiduc, HP InfoTech s.r.l.
http://www.hpinfotech.com

Project : 
Version : 
Date    : 17.03.2010
Author  : hardlock
Company : Hardlock Company
Comments: 


Chip type           : ATtiny13
Clock frequency     : 9,600000 MHz
Memory model        : Tiny
External SRAM size  : 0
Data Stack size     : 16
*****************************************************/

#include <tiny13.h>      
#include <delay.h>        

#define ADC_VREF_TYPE 0x00 
#define U12V 0xDC   
/*
12.5 = 0x00DF // 223
13.0 = 0x00E7 // 231
13.5 = 0x00F5 // 245
14.0 = 0x00FC // 252
*/

#define BUTTON !PINB.4       //
#define DOOR_CLOSED PINB.1   //            

#define DELAY_OFF_IF_DOOR_IS_OPEN 60000    // * 0.01 sek = 10min

typedef enum
{
   MODE_STANDBY = 0,
   MODE_SOFT_START,
   MODE_ON, 
   MODE_WAIT_BEFORE_OFF,
   MODE_SOFT_OFF,
   MODE_SETUP_U_1,
   MODE_SETUP_U_2,
   MODE_SETUP_PAUSE_1,
   MODE_SETUP_PAUSE_2,
   MODE_WAIT_AFTER_SAFE_SHUTDOWN
} tModeInfo;

eeprom char eeMultU = 1;
eeprom char eeOnTime5s = 2;

tModeInfo mode;

unsigned int counter;
unsigned int counter2; 
char blinker; 
bit GoToSleep;      
bit SafeShutdown;
bit SafeShutdownEnabled;

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
{
ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
// Delay needed for the stabilization of the ADC input voltage
delay_us(10);
// Start the AD conversion
ADCSRA|=0x40;
// Wait for the AD conversion to complete
while ((ADCSRA & 0x10)==0);
ADCSRA|=0x10;
return ADCW;
}

// Pin change interrupt service routine
interrupt [PCINT0] void pin_change_isr(void)
{
// Place your code here    
  if (!DOOR_CLOSED)
    if (mode == MODE_STANDBY)
    {     
      OCR0A = 0;
      mode = MODE_SOFT_START; 
      GoToSleep = 0;
      SafeShutdown = 0;        
    }       
    
  if (BUTTON)
  {   
    OCR0A = 0;
    GoToSleep = 0;
    SafeShutdown = 0;        
  }
}

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
// Place your code here   
  int iVoltage; 
  int iLevelOff;
  int delay;    
  
  if (mode == MODE_SOFT_OFF) 
  {
    delay = 1000;  //    ...
  }
  else 
  {
    delay = 376;  // 9,6MHz / 255 (Top Timer0) / 376 = 0.01s timer
  }  
  
  counter++;
  if (counter < delay)
  {
    goto l1;
  }
  else
  {
    counter = 0;
  }                     
  
  if (BUTTON)
  {
    if (mode == MODE_STANDBY)   //  
    {              
        if (eeMultU > 6)
        {
          eeMultU = 0;
        }          
        else
        {
          eeMultU++;
        }
        mode = MODE_SETUP_U_1;
        blinker = 0; 
        TCCR0A=0x83;
        OCR0A = 255;   
    }  
    
    if (mode == MODE_ON)       //        .
    {              
        if (eeOnTime5s > 11)
        {
          eeOnTime5s = 0;
        }          
        else
        {
          eeOnTime5s++;
        }
        mode = MODE_SETUP_PAUSE_1;
        blinker = 0; 
        OCR0A = 0;   
    }
  }
             
  iVoltage = read_adc(1);              //   
  iLevelOff = U12V + (eeMultU * 5);    // ,        
  
  switch (mode) 
  {                    
    case MODE_STANDBY:      // - .
      OCR0A = 0;            //  
      TCCR0A = 0x03;          //  .
      SafeShutdownEnabled = 1;
      SafeShutdown = 0;
      
      GoToSleep = 1;        //  .
    break;
    case MODE_SOFT_START:   // 
      TCCR0A = 0x83;          //  
      if (OCR0A < 255)      //    255
      {
        OCR0A++;            // 
      }
      else
      {             
        mode = MODE_ON;     //    (255),          ""
        counter2 = 0;
      }    
    break;
    case MODE_ON:            // 
      counter2++;
      if (SafeShutdownEnabled)
      {
        if (counter2 > DELAY_OFF_IF_DOOR_IS_OPEN)        //    ,  
        {                                                      
          SafeShutdown = 1;
          mode = MODE_SOFT_OFF;        //  ...
        }
      }
      
      if (DOOR_CLOSED)               //  ...
      {  
        if (iVoltage > iLevelOff)    //..    , 
        {
          mode = MODE_SOFT_OFF;      //   
        }
        else
        {
          mode = MODE_WAIT_BEFORE_OFF;   //   .
          counter2 = 0;
        }
      }
      
    break;
    case MODE_WAIT_BEFORE_OFF:                // (eeOnTime5s * 5)   .
      if (++counter2 > (eeOnTime5s * 500)) 
        mode = MODE_SOFT_OFF;   
      
      if (!DOOR_CLOSED) 
        mode = MODE_ON;   
      
      
      if (iVoltage > iLevelOff) 
      {
        if (DOOR_CLOSED) 
          mode = MODE_SOFT_OFF;
      }
    break;
    case MODE_SOFT_OFF: //
      if (OCR0A > 0)
      {
        OCR0A--;
        if (!DOOR_CLOSED) 
        {
          if (!SafeShutdown)
            mode = MODE_SOFT_START; 
        }  
      }
      else
      {              
        mode = MODE_STANDBY;
        if (SafeShutdown)
          mode = MODE_WAIT_AFTER_SAFE_SHUTDOWN;
        SafeShutdown = 0;
      }
    break;                     
    case MODE_SETUP_U_1:        
      if (++counter2 < 20) break;
      counter2 = 0;
      
      if (blinker > eeMultU)
      {
        blinker++;
        mode = MODE_STANDBY;
      }          
      else
      {               
        TCCR0A = 0x83;
        OCR0A = 255;
        mode = MODE_SETUP_U_2;
      }     
    break;          
    case MODE_SETUP_U_2: 
      if (++counter2 < 50) 
        break;
      counter2 = 0;               
      mode = MODE_SETUP_U_1;
      OCR0A = 0;
    break;         
    case MODE_SETUP_PAUSE_1:        
      if (++counter2 < 50) break;
      
      counter2 = 0;
                         

      if (blinker > eeOnTime5s)
      {     
        blinker++;
        mode = MODE_ON;
      }          
      else
      {               
        OCR0A = 0;
        mode = MODE_SETUP_PAUSE_2;
      }     
    break;    
    case MODE_SETUP_PAUSE_2: 
      if (++counter2 < 50) break;  
      
      counter2 = 0;               
      mode = MODE_SETUP_PAUSE_1;
      OCR0A = 255;
    break;  
    case MODE_WAIT_AFTER_SAFE_SHUTDOWN:
      if (!DOOR_CLOSED)
      {
        mode = MODE_SOFT_START;
        SafeShutdownEnabled = 0;
      }
      
      if (++counter2 < 1000) 
        break;
      counter2 = 0;      
      
      mode = MODE_STANDBY;
    break;     
  }
 
l1:
  
}

// Declare your global variables here

void main(void)
{
// Declare your local variables here

// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Input/Output Ports initialization
// Port B initialization
// Func5=In Func4=In Func3=In Func2=In Func1=In Func0=Out 
// State5=T State4=P State3=P State2=T State1=P State0=0 
PORTB=0x1A;
DDRB=0x01;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 9600,000 kHz
// Mode: Fast PWM top=FFh
// OC0A output: Non-Inverted PWM
// OC0B output: Disconnected
TCCR0A=0x03;
TCCR0B=0x01;
//TCNT0=0x00;
//OCR0A=0x00;
//OCR0B=0x00;

// External Interrupt(s) initialization
// INT0: Off
// Interrupt on any change on pins PCINT0-5: On
GIMSK=0x20;
MCUCR=0x30; //Sleep enabled. Power-down mod
PCMSK=0x12; //Pin Change interupt on PIN.1, PIN.4
GIFR=0x20;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=0x02;

// Analog Comparator initialization
// Analog Comparator: Off
ACSR=0x80;
//ADCSRB=0x00;

// ADC initialization
// ADC Clock frequency: 600,000 kHz
// ADC Bandgap Voltage Reference: Off
// ADC Auto Trigger Source: None
// Digital input buffers on ADC0: On, ADC1: On, ADC2: On, ADC3: On
DIDR0&=0x03;
DIDR0|=0x00;
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x84;               

mode = MODE_STANDBY; 
//if (!DOOR_CLOSED)
//  mode = MODE_SOFT_START;
//GoToSleep = 1;   
//SafeShutdown = 0;

// Global enable interrupts
#asm("sei");       

while (1)
      {
      // Place your code here
        if (GoToSleep)
        {             
          #asm("sei")
          #asm("sleep")
        }
      };
}
